import logging

logger = logging.getLogger(__name__)


class MusesPagesViews:
    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.
        logger.info("Enter in MusesPagesViews")

        if request.user.is_authenticated and request.method == "GET":
            request.user.page_views_counter += 1
            request.user.save()

        response = self.get_response(request)

        # Code to be executed for each request/response after
        # the view is called.

        logger.info("Exit from MusesPagesViews")

        return response



# Create your models here.
import pytz
from django.contrib.auth.base_user import AbstractBaseUser, BaseUserManager
from django.contrib.auth.models import PermissionsMixin, Group
from django.contrib.auth.validators import UnicodeUsernameValidator
from django.core.files.storage import FileSystemStorage
from django.core.mail import send_mail
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from phonenumber_field.modelfields import PhoneNumberField
from precise_bbcode.fields import BBCodeTextField

from db.validators import not_passed
from editorjs.fields import EditorJSField


class BaseModel(models.Model):
    created_at = models.DateTimeField(
        verbose_name=_('created at'),
        unique=False,
        null=False,
        blank=False,
        auto_now_add=True
    )
    updated_at = models.DateTimeField(
        verbose_name=_('updated at'),
        unique=False,
        null=False,
        blank=False,
        auto_now=True
    )
    deleted_at = models.DateTimeField(
        verbose_name=_('deleted at'),
        unique=False,
        null=True,
        blank=True,
        db_index=True,
    )

    class Meta:
        abstract = True


class MemberQuerySet(models.QuerySet):
    def superusers(self):
        return self.filter(is_superuser=True)

    def staff(self):
        return self.filter(is_staff=True)

    def privileges(self):
        return self.filter(status=Member.PRIVILEGE)

    def regulars(self):
        return self.filter(status=Member.REGULAR)


class MemberManager(BaseUserManager):
    def get_queryset(self):
        return MemberQuerySet(self.model, using=self._db)

    def superusers(self):
        return self.get_queryset().superusers()

    def staff(self):
        return self.get_queryset().staff()

    def privileges(self):
        return self.get_queryset().privileges()

    def regulars(self):
        return self.get_queryset().regulars()

    def create_user(self, username, email, date_of_birth, password=None, **kwargs):
        email = MemberManager.normalize_email(email)
        if password is None:
            password = MemberManager.make_random_password(self)
        member = Member(username=username, email=email, date_of_birth=date_of_birth)
        member.set_password(password)
        for key, value in kwargs.items():
            setattr(member, key, value)
        member.clean()
        member.save()

    def create_superuser(self, username, email, date_of_birth, password=None, **kwargs):
        if password is None:
            password = MemberManager.make_random_password(self)
        member = Member(username=username, email=email, date_of_birth=date_of_birth)
        member.is_staff = True
        member.is_superuser = True
        member.set_password(password)
        for key, value in kwargs.items():
            setattr(member, key, value)
        member.clean()
        member.save()


class Member(AbstractBaseUser, PermissionsMixin, BaseModel):
    username_validator = UnicodeUsernameValidator()
    username = models.CharField(
        _('username'),
        max_length=150,
        unique=True,
        help_text=_('Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.'),
        validators=[username_validator],
        error_messages={
            'unique': _("A user with that username already exists."),
        },
    )
    first_name = models.CharField(_('first name'), max_length=30, blank=True)
    last_name = models.CharField(_('last name'), max_length=150, blank=True)
    email = models.EmailField(_('email address'), blank=True)
    date_of_birth = models.DateField()
    is_staff = models.BooleanField(
        _('staff status'),
        default=False,
        help_text=_('Designates whether the user can log into this admin site.'),
    )
    is_active = models.BooleanField(
        _('active'),
        default=True,
        help_text=_(
            'Designates whether this user should be treated as active. '
            'Unselect this instead of deleting accounts.'
        ),
    )
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    def member_avatar_path(self, filename):
        return 'avatars/member_{0}/{1}'.format(self.id, filename)

    avatar = models.ImageField(
        verbose_name=_('avatar'),
        upload_to=member_avatar_path,
        storage=FileSystemStorage
    )
    web_site = models.URLField(
        verbose_name=_('web site'),
        max_length=2048,
    )
    gender = models.CharField(
        verbose_name=_('gender'),
        max_length=1
    )
    occupation = models.CharField(
        verbose_name=_('occupation'),
        max_length=255
    )
    phone_number = PhoneNumberField(
        verbose_name=_('phone number'),
        blank=True
    )
    privilege_start_date = models.DateField(
        verbose_name=_('privilege start date'),
        null=True
    )
    postal_address = models.CharField(
        verbose_name=_('postal address'),
        max_length=1024
    )
    postal_address_2 = models.CharField(
        verbose_name=_('postal address 2'),
        max_length=1024
    )
    postal_city = models.CharField(
        verbose_name=_('postal city'),
        max_length=200
    )
    postal_code = models.CharField(
        verbose_name=_('postal code'),
        max_length=20
    )
    language = models.CharField(
        verbose_name=_('language'),
        max_length=2
    )
    locked = models.BooleanField(
        verbose_name=_('locked'),
        default=False
    )

    """
    ------------------------------
    *** DEBUT LISTE DES THEMES ***
    ------------------------------
    Section à compléter pour ajouter des thèmes complémentaires
    """

    THEME_DEFAULT = "muses_default"
    THEME_CLASSIC = "muses_classic"
    THEME_ROSE = "muses_rose"
    THEME_BLUE = "muses_blue"
    THEME_GREEN = "muses_green"
    THEME_ORANGE = "muses_orange"

    THEME_CHOICES = (
        (THEME_DEFAULT, _("Default")),
        (THEME_CLASSIC, _("Classic")),
        (THEME_ROSE, _("Rose")),
        (THEME_BLUE, _("Blue")),
        (THEME_GREEN, _("Green")),
        (THEME_ORANGE, _("Orange")),
    )

    """
    ------------------------------
    *** FIN LISTE DES THEMES ***
    ------------------------------
    """

    theme = models.CharField(
        verbose_name=_('theme'),
        max_length=30,
        choices=THEME_CHOICES,
        default=THEME_DEFAULT
    )

    connections_counter = models.IntegerField(
        verbose_name=_('connections counter'),
        default=0,
    )
    page_views_counter = models.IntegerField(
        verbose_name=_('page views counter'),
        default=0,
    )

    email_notification_on_new_comment = models.BooleanField(
        verbose_name=_('email notification on new comment'),
        default=False
    )
    email_notification_on_new_message = models.BooleanField(
        verbose_name=_('email notification on new message'),
        default=False
    )

    mailbox_collapsing = models.BooleanField(
        verbose_name=_('mailbox collapsing'),
        default=False
    )
    mailbox_blocked = models.BooleanField(
        verbose_name=_('mailbox blocked'),
        default=False
    )

    ALL_TIMEZONES = sorted((item, item) for item in pytz.all_timezones)
    tz_name = models.CharField(choices=ALL_TIMEZONES, max_length=64)

    story = models.TextField(
        verbose_name=_('story'),
        default="",
        unique=False,
        blank=False,
        null=False
    )

    REGULAR = 'R'
    PRIVILEGE = 'P'
    MEMBER_STATUS_CHOICES = [
        (REGULAR, 'REGULAR'),
        (PRIVILEGE, 'PRIVILEGE')
    ]
    status = models.CharField(
        verbose_name=_('status'),
        max_length=10,
        choices=MEMBER_STATUS_CHOICES,
        default=REGULAR
    )

    USERNAME_FIELD = 'username'
    EMAIL_FIELD = 'email'
    REQUIRED_FIELDS = ['date_of_birth', 'email']

    blocked_contacts = models.ManyToManyField(
        to='self',
        related_name='blocked_contact'
    )

    objects = MemberManager()

    class Meta:
        verbose_name = _('member')
        verbose_name_plural = _('members')

    def clean(self):
        super().clean()
        self.email = self.__class__.objects.normalize_email(self.email)

    def get_full_name(self):
        """
        Return the first_name plus the last_name, with a space in between.
        """
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_short_name(self):
        """Return the short name for the user."""
        return self.first_name

    def email_user(self, subject, message, from_email=None, **kwargs):
        """Send an email to this user."""
        send_mail(subject, message, from_email, [self.email], **kwargs)


class MessageManager(models.Manager):
    pass


class Message(BaseModel):
    subject = models.CharField(
        verbose_name=_('subject'),
        unique=True,
        null=False,
        blank=False,
        db_index=True,
        max_length=255,
        default=_("no subject")
    )
    content = models.TextField(
        verbose_name=_('content'),
        unique=False,
        null=True,
        blank=True,
    )
    sender = models.ForeignKey(
        to=Member,
        related_name='sender_ref',
        on_delete=models.CASCADE
    )
    recipient = models.ManyToManyField(
        to=Member,
        verbose_name=_('recipient'),
        related_name='recipients',
        related_query_name='recipients'
    )
    message_box = models.ForeignKey(
        to=Member,
        related_name=_('message_box'),
        on_delete=models.CASCADE
    )

    def __str__(self):
        return self.subject

    class Meta:
        verbose_name = 'message'
        verbose_name_plural = 'messages'

    def send(self):
        pass


class SiteParamManager(models.Manager):
    pass


class SiteParam(BaseModel):
    key = models.CharField(
        verbose_name=_('key'),
        unique=True,
        null=False,
        blank=False,
        db_index=True,
        max_length=255,
        default=_("no name")
    )
    value = models.CharField(
        verbose_name=_("value"),
        unique=True,
        null=False,
        blank=False,
        default=_('no value'),
        db_index=True,
        max_length=2048
    )

    objects = SiteParamManager()

    class Meta:
        verbose_name = 'site parameter'
        verbose_name_plural = 'site parameters'


class SectionManager(models.Manager):
    pass


class Section(BaseModel):
    ACTIVE = 1
    DISABLED = 2
    STATUS_CHOICES = (
        (ACTIVE, _('active')),
        (DISABLED, _('disabled'))
    )
    short_name = models.CharField(
        verbose_name=_('short name'),
        unique=True,
        max_length=20,
        default=_('no short name'),
        db_index=True
    )
    name = models.CharField(
        verbose_name=_('name'),
        unique=False,
        max_length=255,
        default=_('no name'),
        db_index=True
    )
    order = models.IntegerField(
        verbose_name=_('order'),
        unique=True,
        default=0
    )
    description = models.TextField(
        verbose_name=_('description'),
        max_length=1024,
        null=True
    )
    status = models.IntegerField(
        verbose_name=_('status'),
        choices=STATUS_CHOICES,
        default=DISABLED
    )
    groups = models.ManyToManyField(
        to=Group,
        verbose_name=_('groups'),
        blank=True,
        help_text=_(
            'The groups this section belongs to.'
        ),
        related_name="section_set",
        related_query_name="section",
    )

    class Meta:
        verbose_name = _('section')
        verbose_name_plural = _('sections')


class TagManager(models.Manager):
    pass


class Tag(BaseModel):
    name = models.CharField(
        max_length=255,
        unique=True,
        null=False,
        blank=False,
        db_index=True
    )
    enable_at = models.DateTimeField(
        verbose_name=_('enable at'),
        unique=False,
        null=True,
        blank=True,
        db_index=True
    )
    disable_at = models.DateTimeField(
        verbose_name=_('disable at'),
        unique=False,
        null=True,
        blank=True,
        db_index=True
    )
    CATEGORY = 0
    THEME = 1
    PROSODY = 2
    FORM = 3
    USER = 4
    EVENT = 5
    TYPE_CHOICES = (
        (CATEGORY, _('category')),
        (THEME, _('theme')),
        (PROSODY, _('prosody')),
        (FORM, _('form')),
        (USER, _('user')),
        (EVENT, _('event')),
    )
    type = models.CharField(
        verbose_name=_('type'),
        unique=False,
        null=False,
        blank=True,
        db_index=False,
        max_length=255,
        choices=TYPE_CHOICES,
        default=USER
    )
    mature = models.BooleanField(
        verbose_name=_('mature'),
        unique=False,
        null=False,
        blank=True,
        default=False
    )
    active = models.BooleanField(
        verbose_name=_('active'),
        unique=False,
        null=False,
        blank=False,
        default=False
    )

    valid_sections = models.ManyToManyField(
        to=Section
    )

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = 'tag'
        verbose_name_plural = 'tags'


class AdmonitionTemplateManager(models.Manager):
    pass


class AdmonitionTemplate(BaseModel):
    subject = models.CharField(
        verbose_name=_('subject'),
        unique=True,
        null=False,
        blank=False,
        db_index=True,
        max_length=255,
        default=_("no subject")
    )
    content = models.TextField(
        verbose_name=_('content'),
        unique=False,
        null=True,
        blank=True,
    )
    objects = AdmonitionTemplateManager()


class LicenseManager(models.Manager):
    pass


class License(BaseModel):
    name: str = models.CharField(
        verbose_name=_('name'),
        max_length=255,
        null=True
    )
    text = models.TextField(
        verbose_name=_('text'),
        null=True
    )
    logo = models.ImageField(
        verbose_name=_('logo'),
        null=True
    )
    active = models.BooleanField(
        verbose_name=_('active'),
        unique=False,
        null=False,
        blank=False,
        default=False
    )

    def __str__(self):
        return self.name

    objects = LicenseManager()

    class Meta:
        verbose_name = 'license'
        verbose_name_plural = 'licenses'


class BookManager(models.Manager):
    pass


class Book(BaseModel):
    title = models.CharField(
        verbose_name=_('title'),
        max_length=255
    )
    isbn = models.CharField(
        verbose_name=_('isbn'),
        max_length=20,
    )
    published_at = models.DateField(
        verbose_name=_('published date')
    )
    author = models.ForeignKey(
        to=Member,
        related_name='book_author',
        on_delete=models.CASCADE
    )
    license = models.ForeignKey(
        to=License,
        related_name='book_license',
        on_delete=models.CASCADE,
    )
    visible = models.BooleanField(
        verbose_name=_('visible'),
        unique=False,
        null=False,
        blank=False,
        default=False
    )

    def __str__(self):
        return self.title

    objects = BookManager()

    class Meta:
        verbose_name = 'book'
        verbose_name_plural = 'books'


class BookPartManager(models.Manager):
    pass


class BookPart(BaseModel):
    title = models.CharField(
        verbose_name=_('title'),
        unique=False,
        null=False,
        blank=False,
        db_index=True,
        max_length=255
    )
    order = models.IntegerField(
        verbose_name=_('order'),
        unique=False,
        null=True,
        blank=True,
        db_index=False
    )
    book = models.ForeignKey(
        to=Book,
        related_name='book_ref',
        on_delete=models.CASCADE
    )

    def __str__(self):
        return self.title

    objects = BookPartManager()

    class Meta:
        verbose_name = 'bookpart'
        verbose_name_plural = 'bookparts'


class PostManager(models.Manager):
    pass


class Post(BaseModel):
    old_slug = models.URLField(
        verbose_name=_('old slug'),
        max_length=2048,
    )
    DRAFT = 'draft'
    PUBLISHED = 'published'
    WARNED = 'warned'
    ARCHIVED = 'archived'
    POST_STATUS_CHOICES = [
        (DRAFT, _('DRAFT')),
        (PUBLISHED, _('PUBLISHED')),
        (WARNED, _('WARNED')),
        (ARCHIVED, _('ARCHIVED'))
    ]
    title = models.CharField(
        verbose_name=_('title'),
        unique=False,
        null=False,
        blank=False,
        max_length=255,
    )
    summary = EditorJSField(
        verbose_name=_('summary'),
        unique=False,
        null=True,
        blank=True,
        max_length=2000,
    )
    bbcode_content = BBCodeTextField(
        verbose_name=_('bbcode content'),
        unique=False,
        null=True,
        blank=True,
    )
    content = EditorJSField(
        verbose_name=_('content'),
        unique=False,
        null=True,
        blank=True,
    )
    status = models.CharField(
        verbose_name=_('status'),
        choices=POST_STATUS_CHOICES,
        max_length=20,
        default=DRAFT
    )
    hits_counter = models.IntegerField(
        verbose_name=_('hits count'),
        default=0
    )
    revisions_counter = models.IntegerField(
        verbose_name=_('revisions count'),
        default=0
    )
    validated_at = models.DateTimeField(
        verbose_name=_('validated at'),
        unique=False,
        null=True,
        blank=False
    )
    validated_by = models.ForeignKey(
        to=Member,
        on_delete=models.DO_NOTHING,
        related_name="validated_by",
        null=True,
    )
    author = models.ForeignKey(
        to=Member,
        related_name='post_author',
        on_delete=models.CASCADE
    )
    readings = models.ManyToManyField(
        to=Member
    )
    book_part = models.ForeignKey(
        to=BookPart,
        related_name='book_part_ref',
        on_delete=models.CASCADE,
        null=True,
    )
    tags = models.ManyToManyField(
        to=Tag
    )
    revision = models.ForeignKey(
        to='self',
        related_name='revision_ref',
        on_delete=models.CASCADE,
        null=True
    )
    section = models.ForeignKey(
        to=Section,
        related_name='section_ref',
        on_delete=models.CASCADE
    )

    objects = PostManager()

    class Meta:
        verbose_name = 'post'
        verbose_name_plural = 'posts'


class CommentManager(models.Manager):
    pass


class Comment(BaseModel):
    author = models.ForeignKey(
        to=Member,
        related_name='comment_author',
        on_delete=models.CASCADE
    )
    content = EditorJSField(
        verbose_name=_('content')
    )
    reply_to = models.ForeignKey(
        to='self',
        related_name='comment_reply_to',
        on_delete=models.CASCADE
    )
    readings = models.ManyToManyField(
        to=Member
    )
    objects = CommentManager()


class Rating(models.Model):
    level = models.PositiveSmallIntegerField(
        verbose_name=_('level'),
        unique=False,
        null=False,
        blank=False,
        default=0,
        validators=[
            MaxValueValidator(5),
            MinValueValidator(1)
        ],
    )
    owner = models.ForeignKey(
        verbose_name=_('owner'),
        to=Member,
        on_delete=models.CASCADE,
        null=False,
        blank=False,
    )

    class Meta:
        abstract = True


class PostRatingManager(models.Manager):
    pass


class PostRating(BaseModel, Rating):
    post = models.ForeignKey(
        to=Post,
        on_delete=models.CASCADE
    )
    objects = PostRatingManager()


class CommentRatingManager(models.Manager):
    pass


class CommentRating(BaseModel, Rating):
    comment = models.ForeignKey(
        to=Comment,
        on_delete=models.CASCADE
    )
    objects = CommentRatingManager()


class Like(models.Model):
    level = models.PositiveSmallIntegerField(
        verbose_name=_('level'),
        unique=False,
        null=False,
        blank=False,
        default=0,
        validators=[
            MaxValueValidator(5),
            MinValueValidator(1)
        ],
    )
    owner = models.ForeignKey(
        verbose_name=_('owner'),
        to=Member,
        on_delete=models.CASCADE,
        null=False,
        blank=False,
    )

    class Meta:
        abstract = True


class PostLikeManager(models.Manager):
    pass


class PostLike(BaseModel, Like):
    post = models.ForeignKey(
        to=Post,
        on_delete=models.CASCADE
    )
    objects = PostLikeManager()


class CommentLikeManager(models.Manager):
    pass


class CommentLike(BaseModel, Like):
    comment = models.ForeignKey(
        to=Comment,
        on_delete=models.CASCADE
    )
    objects = CommentLikeManager()


class AdmonitionManager(models.Manager):
    pass


class Admonition(BaseModel):
    subject = models.CharField(
        verbose_name=_('subject'),
        unique=True,
        null=False,
        blank=False,
        db_index=True,
        max_length=255,
        default=_("no subject")
    )
    content = models.TextField(
        verbose_name=_('content'),
        unique=False,
        null=True,
        blank=True,
    )
    objects = AdmonitionManager()


class AdmonitionActionManager(models.Manager):
    pass


class AdmonitionAction(BaseModel):
    SYSTEM_ACTION = 'systemAction'
    MESSAGE_TO_USER = 'msgToUser'
    MESSAGE_FROM_USER = 'msgFromUser'
    INTERNAL_MESSAGE = 'internalMsg'
    TYPE_CHOICES = (
        (SYSTEM_ACTION, _('System Action')),
        (MESSAGE_FROM_USER, _('Message from member')),
        (MESSAGE_TO_USER, _('Message to member')),
        (INTERNAL_MESSAGE, _('Internal message'))
    )
    message = models.TextField(
        verbose_name=_('message'),
    )
    type = models.CharField(
        verbose_name=_('type'),
        max_length=30,
        choices=TYPE_CHOICES,
        default=SYSTEM_ACTION
    )
    admonition = models.ForeignKey(
        to=Admonition,
        on_delete=models.CASCADE
    )
    objects = AdmonitionActionManager()


class Alert(models.Model):
    PLAGIARISM = 'plagiarism'
    SPELLING = 'spelling'
    BAD_CATEGORY = 'bad_category'
    HATE = 'hate'
    OTHER = 'other'
    TYPE_ALERT_CHOICES = (
        (PLAGIARISM, _('Plagiarism')),
        (SPELLING, _('Spelling')),
        (BAD_CATEGORY, _('Bad category')),
        (HATE, _('Hate')),
        (OTHER, _('Other'))
    )

    OPEN = 1
    CLOSED = 2
    DISCUSSION = 3
    STATUS_CHOICES = (
        (OPEN, _('open')),
        (CLOSED, _('closed')),
        (DISCUSSION, _('discussion'))
    )

    type = models.CharField(
        verbose_name=_('type'),
        max_length=30,
        choices=TYPE_ALERT_CHOICES,
        default=OTHER
    )
    details = models.TextField(
        verbose_name=_('details'),
        max_length=1000,
        default=""
    )
    status = models.IntegerField(
        verbose_name=_('status'),
        default=1
    )

    class Meta:
        abstract = True


class PostAlertManager(models.Manager):
    pass


class PostAlert(BaseModel, Alert):
    post = models.ForeignKey(
        to=Post,
        on_delete=models.CASCADE
    )

    objects = PostAlertManager()


class CommentAlertManager(models.Manager):
    pass


class CommentAlert(BaseModel, Alert):
    comment = models.ForeignKey(
        to=Comment,
        on_delete=models.CASCADE
    )

    objects = CommentAlertManager()


class ContestManager(models.Manager):
    pass


class Contest(BaseModel):
    name = models.CharField(
        verbose_name=_('name'),
        unique=False,
        null=False,
        blank=False,
        max_length=255,
        db_index=True
    )
    starts_at = models.DateField(
        verbose_name=_('start at'),
        unique=False,
        null=False,
        blank=False,
        db_index=True,
        validators=[not_passed]
    )
    ends_at = models.DateField(
        verbose_name=_('ends at'),
        unique=False,
        null=False,
        blank=False,
        db_index=True,
        validators=[not_passed]
    )
    objects = ContestManager()


class CorrectionRequestManager(models.Manager):
    pass


class CorrectionRequest(BaseModel):
    title = models.CharField(
        verbose_name=_('title'),
        unique=False,
        null=False,
        blank=False,
        max_length=255,
    )
    summary = EditorJSField(
        verbose_name=_('summary'),
        unique=False,
        null=True,
        blank=True,
        max_length=2000,
    )
    bbcode_content = BBCodeTextField(
        verbose_name=_('bbcode content'),
        unique=False,
        null=True,
        blank=True,
    )
    content = EditorJSField(
        verbose_name=_('content'),
        unique=False,
        null=True,
        blank=True,
    )
    post = models.ForeignKey(
        to=Post,
        on_delete=models.DO_NOTHING
    )
    objects = CorrectionRequestManager()


class CorrectionRequestActionManager(models.Manager):
    pass


class CorrectionRequestAction(BaseModel):
    CORRECTION = 1
    VALIDATION = 2
    ACTION_CHOICES = (
        (CORRECTION, _('Correction')),
        (VALIDATION, _('Validation'))
    )
    owner = models.ForeignKey(
        to=Member,
        on_delete=models.DO_NOTHING
    )
    correction_request = models.ForeignKey(
        to=CorrectionRequest,
        on_delete=models.DO_NOTHING
    )
    action = models.CharField(
        verbose_name=_('action'),
        max_length=30,
        choices=ACTION_CHOICES,
        default=CORRECTION
    )

    objects = CorrectionRequestActionManager()

    class Meta:
        verbose_name = 'Correction Request'
        verbose_name_plural = 'Correction Requests'


class PostListManager(models.Manager):
    pass


class PostList(BaseModel):
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255,
        null=True
    )
    owner = models.ForeignKey(
        to=Member,
        on_delete=models.CASCADE
    )
    posts = models.ManyToManyField(
        to=Post
    )
    removable = models.BooleanField(
        verbose_name=_('removable'),
        default=True
    )
    public = models.BooleanField(
        verbose_name=_('public'),
        default=False
    )

    FAVOURITE_TYPE = 1
    READING_LIST_TYPE = 2
    TYPE_CHOICES = (
        (FAVOURITE_TYPE, _('Favourite')),
        (READING_LIST_TYPE, _('Reading'))
    )
    type = models.IntegerField(
        verbose_name=_('type'),
        choices=TYPE_CHOICES,
        default=READING_LIST_TYPE
    )

    objects = PostListManager()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = 'Post List'
        verbose_name_plural = 'Post Lists'


class FaqManager(models.Manager):
    pass


class Faq(BaseModel):
    DRAFT = 'draft'
    PUBLISHED = 'published'
    FAQ_STATUS_CHOICES = [
        (DRAFT, _('DRAFT')),
        (PUBLISHED, _('PUBLISHED')),
    ]

    order = models.IntegerField(
        verbose_name=_('order')
    )
    question = models.CharField(
        verbose_name=_('question'),
        max_length=2048,
        null=True
    )
    answer = models.TextField(
        verbose_name=_('answer')
    )

    objects = FaqManager()

    def __str__(self):
        return self.id

    class Meta:
        verbose_name = 'Faq'
        verbose_name_plural = 'Faqs'


class VotingManager(models.Manager):
    pass


class Voting(BaseModel):
    name = models.CharField(
        verbose_name=_('name'),
        max_length=200
    )

    objects = VotingManager()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = 'Voting'
        verbose_name_plural = 'Votings'


class VotingRoundManager(models.Manager):
    pass


class VotingRound(BaseModel):
    order = models.IntegerField(
        verbose_name=_('order'),
        unique=True,
        null=False,
        blank=False
    )
    voting = models.ForeignKey(
        to=Voting,
        on_delete=models.CASCADE
    )

    objects = VotingRoundManager()

    def __str__(self):
        return self.id

    class Meta:
        verbose_name = 'Voting Round'
        verbose_name_plural = 'Voting Rounds'


class VoteManager(models.Manager):
    pass


class Vote(BaseModel):
    round = models.IntegerField(
        verbose_name=_('round')
    )
    owner = models.ForeignKey(
        to=Member,
        on_delete=models.DO_NOTHING
    )
    post = models.ForeignKey(
        to=Post,
        on_delete=models.DO_NOTHING
    )
    voting_round = models.ForeignKey(
        to=VotingRound,
        on_delete=models.DO_NOTHING
    )

    objects = VoteManager()

    def __str__(self):
        return self.id

    class Meta:
        verbose_name = 'Vote'
        verbose_name_plural = 'Votes'



# Create your views here.
from django.http import HttpResponse, HttpResponseNotAllowed
from django.shortcuts import redirect
from django.utils.translation import ugettext as _

from .generic import DetailView, TemplateView, ListView, FormView, CreateView, UpdateView, AdminListView, \
    AdminCreateView, AdminDetailView, AdminDeleteView, ModListView, ModDetailView, ModCreateView, ModDeleteView, \
    ModUpdateView, AdminUpdateView
from db.models import Post, Member, Tag, Book, BookPart, SiteParam, Admonition, Faq, License, CorrectionRequest, \
    Contest, CorrectionRequestAction, PostAlert, CommentAlert, Message, Comment
from web.forms import LoginForm, RegisterForm, ContactForm, ControlCenterChangeEmailForm, \
    PostForm, \
    CommentForm, ControlCenterChangePasswordForm, ProfileGeneralInformationForm, PreferencesNotificationsForm, Form, \
    LogoutForm, TagForm, LicenseForm, PreferencesGeneralForm
from .generic.cor_detail_view import CorDetailView
from .generic.cor_list_view import CorListView


class MultiFormsView(TemplateView):
    form_classes = {}

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        for key, value in self.form_classes.items():
            context[key] = value(id_form=key)
        return context


def save_selected_theme(request):
    if request.method == 'POST':
        current_user = Member.objects.get(id=request.user.id)
        current_user.theme = request.POST['theme']
        current_user.save()
        return redirect(request.POST['target_url'])
    else:
        return HttpResponseNotAllowed(permitted_methods=['POST'])


def change_password(request):
    if request.method == 'POST':
        current_user = Member.objects.get(id=request.user.id)
        current_user.set_password(request.POST["password"])
        return redirect(request.POST['target_url'])
    else:
        return HttpResponseNotAllowed(permitted_methods=['POST'])


class MemberDetailView(DetailView):
    model = Member


class ContactPageView(FormView):
    form_class = ContactForm
    success_url = '/'

    def form_valid(self, form):
        form.send_message()
        return super().form_valid(form)


class LoginPageView(FormView):
    template_name = "web/login.html"
    section_name = _('Login')
    form_class = LoginForm
    success_url = '/'

    def form_valid(self, form):
        form.authenticate(self.request)
        return super().form_valid(form)


class LogoutPageView(FormView):
    template_name = "web/logout.html"
    section_name = _('Logout')
    form_class = LogoutForm
    success_url = '/'

    def form_valid(self, form):
        form.logout(self.request)
        return super().form_valid(form)


class RegisterPageView(FormView):
    template_name = "web/register.html"
    section_name = _('Register')
    form_class = RegisterForm
    success_url = '/'

    def form_valid(self, form):
        form.register()
        return super().form_valid(form)


class ActivateAccountPageView(TemplateView):
    template_name = "web/activate_account.html"
    section_name = _('Activate Account')


class PostsListView(ListView):
    model = Post
    template_name = "db/post_list.html"
    paginate_by = 10


class PostDetailView(DetailView):
    def get_context_data(self, **kwargs):
        return super().get_context_data(**kwargs)

    template_name = "web/muses_post_detail.html"
    model = Post


class PostCreateView(CreateView):
    def get(self, request, *args, **kwargs):
        return super().get(request, *args, **kwargs)

    template_name = "web/muses_post_create.html"
    model = Post
    form_class = PostForm


class CommentDetailView(DetailView):
    def get_context_data(self, **kwargs):
        return super().get_context_data(**kwargs)

    template_name = "web/muses_comment_detail.html"
    model = Comment


class CommentCreateView(CreateView):
    template_name = "web/muses_comment_create.html"
    model = Comment
    form_class = CommentForm


class AdminMembersListView(ListView):
    template_name = "db/member_admin-list.html"
    model = Member
    paginate_by = 10


class AdminMemberDetailView(DetailView):
    def get_context_data(self, **kwargs):
        return super().get_context_data(**kwargs)

    template_name = "web/muses_admin_member_detail.html"
    model = Member


class AdminPostAlertsListView(ListView):
    template_name = "web/muses_admin_list_post_alerts.html"
    model = PostAlert
    paginate_by = 10


class AdminPostAlertDetailView(DetailView):
    model = PostAlert


class AdminCommentAlertsListView(ListView):
    template_name = "web/muses_admin_list_comment_alerts.html"
    model = CommentAlert
    paginate_by = 10


class AdminCommentAlertDetailView(DetailView):
    model = CommentAlert


class AdminTagsListView(AdminListView):
    model = Tag


class AdminTagDetailView(AdminDetailView):
    model = Tag


class AdminTagCreateView(AdminCreateView):
    model = Tag
    fields = [
        "name",
        "enable_at",
        "disable_at",
        "type",
        "mature",
        "active"
    ]


class AdminTagUpdateView(UpdateView):
    template_name_suffix = "_admin-update"

    model = Tag
    fields = [
        "name",
        "enable_at",
        "disable_at",
        "type",
        "mature",
        "active"
    ]


class AdminTagDeleteView(AdminDeleteView):
    model = Tag


class AdminBooksListView(ListView):
    model = Book
    paginate_by = 10


class AdminBookDetailView(DetailView):
    model = Book


class AdminBookPartsListView(ListView):
    model = BookPart
    paginate_by = 10


class AdminBookPartDetailView(DetailView):
    model = BookPart


class AdminSiteParamsListView(ListView):
    template_name = "web/muses_admin_list_site_params.html"
    model = SiteParam
    paginate_by = 10


class AdminSiteParamDetailView(DetailView):
    model = SiteParam


class AdminAdmonitionDetailView(DetailView):
    model = Admonition


class AdminFaqsListView(ListView):
    template_name = "web/muses_admin_list_faqs.html"
    paginate_by = 10
    model = Faq


class AdminFaqDetailView(DetailView):
    template_name = "web/muses_admin_faq_detail.html"


class AdminCorrectionRequestsListView(ListView):
    template_name = "web/muses_admin_list_correction_requests.html"
    model = CorrectionRequest
    paginate_by = 10


class AdminCorrectionRequestDetailView(DetailView):
    pass


class AdminContestsListView(ListView):
    template_name = "web/muses_admin_list_contests.html"
    model = Contest
    paginate_by = 10


class AdminContestDetailView(DetailView):
    pass


class AdminCorrectionActionsListView(ListView):
    template_name = "web/muses_admin_list_correction_actions.html"
    model = CorrectionRequestAction
    paginate_by = 10


class AdminCorrectionActionDetailView(DetailView):
    pass


class PersonalPostAlertsListView(ListView):
    template_name = "web/muses_personal_list_post_alerts.html"
    model = PostAlert
    paginate_by = 10


class PersonalPostAlertDetailView(DetailView):
    pass


class PersonalAdmonitionDetailView(DetailView):
    pass


class PersonalAdmonitionsListView(ListView):
    template_name = "web/muses_personal_list_admonitions.html"
    model = Admonition
    paginate_by = 10


class PersonalMessagesListView(ListView):
    section_name = _("My Messages")
    template_name = "web/muses_personal_list_messages.html"
    model = Message
    paginate_by = 10

    def get_context_data(self, *, object_list=None, **kwargs):
        context = super().get_context_data(object_list=object_list, **kwargs)
        return context


class PersonalMessageDetailView(DetailView):
    pass


class PersonalControlCenterView(MultiFormsView):
    section_name = _("Control Center")

    def dispatch(self, request, *args, **kwargs):
        if request.method == 'POST':
            form = Form(request.POST)
            return HttpResponse("ok")
        else:
            return super().dispatch(request, *args, **kwargs)

    template_name = "web/muses_personal_control_center.html"
    form_classes = {
        'form_change_password': ControlCenterChangePasswordForm,
        'form_change_email': ControlCenterChangeEmailForm,
    }


class PersonalPreferencesView(MultiFormsView):
    template_name = "web/muses_personal_preferences.html"
    section_name = _("Preferences")
    form_classes = {
        'form_preferences_notifications': PreferencesNotificationsForm,
        'form_preferences_general': PreferencesGeneralForm,
    }


class PersonalProfileView(MultiFormsView):
    template_name = "web/muses_personal_profile.html"
    section_name = "Profile"
    form_classes = {
        'form_change_general_information': ProfileGeneralInformationForm,
    }


class ModMembersListView(ModListView):
    model = Member


class ModMemberDetailView(ModDetailView):
    model = Member


class ModPostAlertsListView(ModListView):
    model = PostAlert


class ModPostAlertDetailView(ModDetailView):
    model = PostAlert


class ModAdmonitionsListView(ModListView):
    model = Admonition


class ModAdmonitionDetailView(ModDetailView):
    model = Admonition


class ModBooksListView(ModListView):
    model = Book


class ModBookDetailView(ModDetailView):
    model = Book


class ModBookPartsListView(ModListView):
    model = BookPart


class ModBookPartDetailView(ModDetailView):
    model = BookPart


class ModSiteParamsListView(ModListView):
    model = SiteParam


class ModSiteParamDetailView(ModDetailView):
    model = SiteParam


class ModFaqsListView(ModListView):
    model = Faq


class ModFaqDetailView(ModDetailView):
    model = Faq


class ModTagsListView(ModListView):
    model = Tag


class ModTagCreateView(ModCreateView):
    model = Tag
    form_class = TagForm


class ModTagUpdateView(ModUpdateView):
    model = Tag
    form_class = TagForm


class ModTagDeleteView(ModDeleteView):
    model = Tag
    form_class = TagForm


class ModLicensesListView(ModListView):
    model = License


class ModLicenseCreateView(ModCreateView):
    model = License


class ModLicenseDeleteView(ModDeleteView):
    model = License


class ModLicenseDetailView(ModDetailView):
    model = License


class ModContestsListView(ModListView):
    model = Contest


class ModContestDetailView(ModDetailView):
    model = Contest


class ModCorrectionRequestsListView(ModListView):
    model = CorrectionRequest


class ModCorrectionRequestDetailView(ModDetailView):
    model = CorrectionRequest


class ModCorrectionActionsListView(ModListView):
    model = CorrectionRequestAction


class ModCorrectionActionDetailView(ModDetailView):
    model = CorrectionRequestAction


class CorrectionPageView(TemplateView):
    template_name = "web/cor.html"


class CorCorrectionRequestsListView(CorListView):
    model = CorrectionRequest


class CorCorrectionRequestDetailView(CorDetailView):
    model = CorrectionRequest


class CorCorrectionActionsListView(CorListView):
    model = CorrectionRequestAction


class CorCorrectionActionDetailView(CorDetailView):
    model = CorrectionRequestAction


class AdminLicenseUpdateView(AdminUpdateView):
    model = License
    form_class = LicenseForm



from urllib.parse import unquote

from django.db.models import Field

from editorjs.widgets import EditorJsWidget


class EditorJSField(Field):
    def __init__(self, *args, **kwargs):
        super(EditorJSField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return 'TextField'

    def clean(self, value, model_instance):
        if value is not None:
            return unquote(super(EditorJSField, self).clean(value, model_instance))
        else:
            return None

    def formfield(self, *args, **kwargs):
        kwargs['widget'] = EditorJsWidget()
        return super(EditorJSField, self).formfield(*args, **kwargs)



from db.models import AdmonitionAction
from web.forms import ModelForm


class AdmonitionActionForm(ModelForm):
    class Meta:
        model = AdmonitionAction
        fields = [
            "message",
        ]



from django.forms import widgets
from django.conf import settings
from django.template.loader import render_to_string


class EditorJsWidget(widgets.TextInput):
    class Media:
        js = (
            'editorjs/editor.js',
            'editorjs/checklist.js',
            'editorjs/delimiter.js',
            'editorjs/header.js',
            'editorjs/image.js',
            'editorjs/inline-code.js',
            'editorjs/link.js',
            'editorjs/list.js',
            'editorjs/paragraph.js',
            'editorjs/quote.js',
            'editorjs/raw.js',
            'editorjs/table.js',
            'editorjs/editorjs_field.js',
        )
        css = {'all': ('editorjs/editorjs_field.css',)}

    def __init__(self, *args, **kwargs):
        super(EditorJsWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, **kwargs):
        elements = {'data_ejs_checklist': kwargs.get('checklist', "true"),
                    'data_ejs_delimiter': kwargs.get('delimiter', "true"),
                    'data_ejs_header': kwargs.get('header', "true"),
                    'data_ejs_image': kwargs.get('image', "true"),
                    'data_ejs_inline-code': kwargs.get('inline-code', "true"),
                    'data_ejs_link': kwargs.get('link', "true"),
                    'data_ejs_list': kwargs.get('list', "true"),
                    'data_ejs_paragraph': kwargs.get('paragraph', "true"),
                    'data_ejs_quote': kwargs.get('quote', "true"),
                    'data_ejs_raw': kwargs.get('raw', "true"),
                    'data_ejs_table': kwargs.get('table', "true")}
        ctx = {
            'name': name,
            'id': kwargs['attrs']['id'],
            'value': value,
            'media_url': settings.MEDIA_URL,
            'attributes': elements
        }

        return render_to_string('editorjs_widget.html', ctx)



from django.http import Http404
from django.utils.decorators import classonlymethod
from django.views.generic import DetailView as DV
from django.utils.translation import gettext as _


class DetailView(DV):
    template_name_suffix = "_detail"
    section_name = ""

    @classonlymethod
    def as_view(cls, **initkwargs):
        if initkwargs.get("model"):
            cls.model = initkwargs.get("model")
        return super().as_view(**initkwargs)

    def get_object(self, queryset=None):
        if queryset is None:
            queryset = self.get_queryset()
        identifier = self.kwargs.get("id")
        if identifier is not None:
            queryset = queryset.filter(id=identifier)
        try:
            obj = queryset.get()
        except queryset.model.DoesNotExist:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj



from db.models import Post
from web.forms import ModelForm


class PostForm(ModelForm):
    class Meta:
        model = Post
        fields = [
            "title",
            "summary",
            "content"
        ]



from django import forms
from django.utils.translation import gettext_lazy as _

from web.forms import Form
from web.forms.widgets import TextInput, Textarea


class ContactForm(Form):
    subject = forms.CharField(
        max_length=60,
        widget=TextInput,
        label=_('Subject')
    )
    content = forms.CharField(
        widget=Textarea,
        label=_('Message')
    )

    def send_message(self):
        pass



"""
Django settings for core project.

Generated by 'django-admin startproject' using Django 3.0.

For more information on this file, see
https://docs.djangoproject.com/en/3.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.0/ref/settings/
"""

import os

from django.utils.translation import gettext_lazy as _
# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
from ruamel.yaml import YAML

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
ENVIRONMENT = os.getenv("ENVIRONMENT", "LOCAL")
VARS_PATH = os.getenv("VARS_PATH", BASE_DIR + "/vars_path")
VARS_PATH_FILENAME = os.getenv("VARS_PATH_FILENAME", "secrets.yml")
with open(f"{VARS_PATH}{os.path.sep}{VARS_PATH_FILENAME}", "r") as secrets_file:
    secrets_content = secrets_file.read()
    yaml = YAML(typ="safe")
    SECRETS_SAFE = yaml.load(secrets_content)

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = SECRETS_SAFE.get("secret_key", "mysuperkey")

# SECURITY WARNING: don't run with debug turned on in production!
if ENVIRONMENT in ['STG', 'PRD']:
    DEBUG = False
else:
    DEBUG = True

ALLOWED_HOSTS = SECRETS_SAFE.get("allowed_hosts", [])

# Application definition

INSTALLED_APPS = [
    'db.apps.DbConfig',
    'api.apps.ApiConfig',
    'web.apps.WebConfig',
    'editorjs.apps.EditorjsConfig',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    # 'debug_toolbar',
    'phonenumber_field',
    'graphene_django',
    'precise_bbcode',
    'qr_code',
    'django_otp',
    'django_otp.plugins.otp_totp',
    'django_otp.plugins.otp_hotp',
    'django_otp.plugins.otp_static',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django_otp.middleware.OTPMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    # 'debug_toolbar.middleware.DebugToolbarMiddleware',
    'web.middlewares.MusesParams',
    'web.middlewares.MusesPagesViews'
]

ROOT_URLCONF = 'core.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            BASE_DIR + "/templates"
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'web.context_processors.global_context',
                'web.context_processors.sections'
            ],
        },
    },
]

WSGI_APPLICATION = 'core.wsgi.application'

# Database
# https://docs.djangoproject.com/en/3.0/ref/settings/#databases

if ENVIRONMENT in ['CI']:
    """ TODO: """
elif ENVIRONMENT in ['DEV', 'STG', 'PRD']:
    """ TODO: """
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

# Authentication User Model
AUTH_USER_MODEL = "db.Member"

# Password validation
# https://docs.djangoproject.com/en/3.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

# Internationalization
# https://docs.djangoproject.com/en/3.0/topics/i18n/

LOCALE_PATHS = [
    BASE_DIR + '/api/locale',
    BASE_DIR + '/core/locale',
    BASE_DIR + '/db/locale',
    BASE_DIR + '/web/locale'
]

LANGUAGES = [
    ('fr', _('French')),
    ('fr-ca', _('Canadian French')),
    ('en-us', _('English'))
]

LANGUAGE_CODE = SECRETS_SAFE.get("language_code", "fr")

TIME_ZONE = SECRETS_SAFE.get("time_zone", "Europe/Paris")

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.0/howto/static-files/

STATIC_URL = SECRETS_SAFE.get("static_url", "http://127.0.0.1:8001/")
STATIC_ROOT = SECRETS_SAFE.get("staticfiles_path", f"{BASE_DIR}/static/")
MEDIA_URL = SECRETS_SAFE.get("media_url", "http://127.0.0.1:8002/")
MEDIA_ROOT = SECRETS_SAFE.get("media_path", f"{BASE_DIR}/media/")

GRAPHENE = {
    'SCHEMA': 'core.schema.schema'
}

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '{levelname} {asctime} {module} {process:d} {thread:d} {message}',
            'style': '{',
        },
        'simple': {
            'format': '{levelname} {message}',
            'style': '{',
        },
    },
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        },
        'file': {
            'level': 'DEBUG',
            'class': 'logging.FileHandler',
            'filename': f"{BASE_DIR}/logs/debug.log",
            'formatter': 'verbose'
        },
    },
    'loggers': {
        'django.db.backends.schema': {
            'handlers': [
                'console'
            ],
            'propagate': True,
            'level': 'INFO',
        },
        'django.request': {
            'handlers': [
                'console'
            ],
            'level': 'ERROR',
            'propagate': False,
        },
        'django': {
            'handlers': [
                'console',
                # 'file'
            ],
            'level': 'ERROR',
            'propagate': True,
        },
    },
}

MUSES = {
    "ADMINISTRATORS_GROUP": "muses_administrators",
    "MODERATORS_GROUP": "muses_moderators",
    "CORRECTORS_GROUP": "muses_correctors"
}

GOOGLE_CLIENT_ID = ""

# SHOW_TOOLBAR_CALLBACK = 'debug_toolbar.middleware.show_toolbar'
#
# INTERNAL_IPS = [
#     '127.0.0.1',
# ]

EMAIL_HOST = SECRETS_SAFE.get("smtp_server", "localhost")
EMAIL_PORT = SECRETS_SAFE.get("smtp_port", 25)
EMAIL_HOST_USER = SECRETS_SAFE.get("smtp_user", "")
EMAIL_HOST_PASSWORD = SECRETS_SAFE.get("smtp_password", "")
EMAIL_USE_TLS = SECRETS_SAFE.get("use_tls", False)
EMAIL_USE_SSL = SECRETS_SAFE.get("use_ssfreel", False)
